﻿using Kivii;
using Kivii.Finances.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Njzj.Ngstc
{
    [Api(Description = "珠宝结算明细接口")]
    public class NgstcGetDetails : RestfulExecution<NgstcDetail>
    {
        public string NgstcId { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            NgstcId.ThrowIfNullOrEmpty("请传入账单编号!");
            var results = NgstcId.GetNgstcDetails();
            var rtns = new RestfulQueryResponse<NgstcDetail>();
            rtns.Results = new List<NgstcDetail>();
            if (!results.IsNullOrEmpty()) rtns.Results.AddRange(results);
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }

    [Api(Description = "珠宝结算查看接口")]
    public class NgstcDetailRead : RestfulExecution<NgstcDetail>
    {
        public Guid Kvid { get; set; }
        public bool IncludeDetail { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            Kvid.ThrowIfEmpty("请传入要查询的结算信息");
            var rtns = new RestfulReadResponse<Settlement>();
            rtns.Result = new Settlement();

            var conn = KiviiContext.GetOpenedDbConnection<Settlement>();
            var settlement = conn.SingleById<Settlement>(Kvid);
            if (settlement != null) rtns.Result.PopulateWith(settlement);
            if (IncludeDetail)
            {
                if (settlement.BizType != typeof(NgstcDetail).FullName) throw new Exception("请传入珠宝系统结算信息！");
                var ngstcDetails = settlement.BizId.GetNgstcDetails();
                var details = ngstcDetails.ConvertToSettlementDetail();
                rtns.Result.Details = new List<SettlementDetail>();
                rtns.Result.Details.AddRange(details);
            }

            return rtns;

        }
    }

    [Api(Description = "珠宝账单回传")]
    public class NgstcUpdateBill : RestfulExecution<NgstcUpdate>
    {
        public NgstcUpdate NgstcUpdate { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            //Settlement item= new Settlement();
            //NgstcUpdate order = new NgstcUpdate();
            //order.BillNo = item.BizId;
            //order.UnPaid = (item.Amount - item.AmountPayment) < 0 ? 0 : item.Amount - item.AmountPayment;
            //order.Paid = item.AmountPayment < 0 ? 0 : item.AmountPayment;
            //order.BillState = "已结账";
            //order.PayDate = item.PayedTime;
            NgstcUpdate.ThrowIfNull("请传入账单信息!");
            var results = NgstcUpdate.UpdateNgstcBill();
            var rtns = new RestfulQueryResponse<NgstcUpdate>();
            rtns.Results = new List<NgstcUpdate>();
            if (!results.IsNullOrEmpty()) rtns.Results.Add(NgstcUpdate);
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }

    [Api(Description = "珠宝系统结算接口")]
    public class SettlementNgstc : RestfulExecution<Settlement>
    {
        public Settlement Item { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            Item.ThrowIfNull("请传入结算信息!");
            Item.PayerName.ThrowIfNullOrEmpty("付款单位不能为空!");
            Item.BizId.ThrowIfNullOrEmpty("请传入唯一性收款单号");
            (Item.Amount <= 0).ThrowIfTrue("核算现价不能小于0");
            (Item.AmountPlan <= 0).ThrowIfTrue("初始价不能小于0");
            Item.OwnerName.ThrowIfNullOrEmpty("请传入实验室/中心信息");
            Item.OperatorName.ThrowIfNullOrEmpty("请传入结算人");

            var conn = KiviiContext.GetOpenedDbConnection<Settlement>();

            if (conn.Exists<Settlement>(o => o.OffsetKvid == Guid.Empty && o.BizId == Item.BizId)) throw new Exception("存在重复结算的对象!");
            var depart = Item.OwnerName.GetOrganization();
            if (depart == null) throw new Exception("未找到部门信息!");
            if (depart != null)
            {
                Item.OwnerName = depart.FullName;
                Item.OwnerKvid = depart.Kvid;
            }
            var kvid = Guid.NewGuid();
            Item.Kvid = kvid;
            Item.Type = Item.Type.IsNullOrEmpty() ? "珠宝结算" : Item.Type;
            Item.Category = "珠宝结算";
            Item.OperateTime = Item.OperateTime == DateTime.MinValue ? DateTime.Now : Item.OperateTime;
            Item.PayeeName = "珠宝系统结算";
            //Item.PayeeName = Item.PayeeName.IsNullOrEmpty() ? "珠宝系统结算" : Item.PayeeName;
            Item.AmountPayment = 0;
            Item.BizType = typeof(NgstcDetail).FullName;
            Item.Currency = CurrencyUnit.CNY;

            var detail = new EntitySettlementDetail<Settlement>();

            detail.SettlementKvid = kvid;
            detail.Type = "珠宝明细";
            detail.Quantity = 1;
            detail.QuantityUnit = "次";
            detail.Currency = Item.Currency;
            detail.Amount = Item.Amount;
            detail.AmountPlan = Item.AmountPlan;
            detail.BizId = Item.BizId;
            detail.BizType = typeof(NgstcDetail).FullName;
            detail.GoodsFullName = "珠宝结算明细";
            detail.Remark = "具体结算明细请查看珠宝系统!";

            var rtns = new RestfulQueryResponse<Settlement>();
            rtns.Results = new List<Settlement>();

            var trans = conn.OpenTransaction();
            try
            {
                conn.Insert(Item);
                rtns.Results.Add(Item);
                conn.Insert(detail);

                trans.Commit();
            }
            catch (Exception ex)
            {
                trans.Rollback();
                throw ex;
            }

            return rtns;
        }
    }
}
